package Modeler;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import selector.caracteristicas.Selector;
import Utilities.ExecuteScript;

/**
 * Clase CreateModels,
 * 
 * Se encarga de generar los 3 modelos
 * svm, red bayesiana y red neuronal
 * @author alejandro
 *
 */

public class CreateModels implements CreateModel{
	
	
	//Variable que recoge las trazas de la clase PCAanalisis
	static Logger logger = Logger.getLogger(CreateModels.class.getName());
	
	//Variable que se encarga de obtener los pesos del analisis de la firma y de las palabras funcionales
	Selector selector;
	
	//Variable que se encarga de interactuar con el entorno R
	ExecuteScript script;
	
	//Variable que almacena los pesos una vez pasado el analisis PCA
	List<Float> pesosPCA = new ArrayList<Float>();
 	
	//Numero de variables usando en cada observacion
	final int NUM_VARIABLES;
	
	int K;
	
	int num_firma;
	int num_function_words;
	//Contructor de la clase PCAanalisis, inicializa las variables
	public CreateModels(int num_firma,int num_function_words ,int numTotal,int K){
		
		System.out.println("Inicializando Selector de caracteristicas...");
	    selector = new Selector(num_firma,num_function_words);
		this.NUM_VARIABLES = numTotal;
		this.K = K;
		this.num_firma = num_firma;
		this.num_function_words = num_function_words;
		System.out.println("Selector de caracteristicas cargado");
		System.out.println("Inicializando el entorno R...");
		script =new ExecuteScript();
		System.out.println("Entorno R cargado correctamente...");		
		
		
	}
	
	/**
	 * Funcion generateModels,
	 * Crea los modelos para los autores
	 * pasados como parámetros
	 * @param nombres
	 * @return
	 */
	public boolean generateModels(String ... nombres){
		
		String path = new File("").getAbsolutePath();
		File directorio;
		List<Float> pesosTrain = new ArrayList<Float>();
		List<Float> pesosTest = new ArrayList<Float>();
		int contador=0;
		BufferedWriter writer_train;
		BufferedWriter writer_test;
		String autores;
		Iterator<Float> it;
		Iterator<String> it2;
		//Si ha introducido menos de dos nombres no se generan los modelos
		if(nombres.length<2){
			logger.severe("Ha introducido menos de dos autores para generar modelos");
			return false;
		}else{
			
			System.out.println("INICIO PROCESO DE CREACION DE LOS MODELOS");
			
			//Creo cabecera de ficheros de pesos
			try {
				new File(path+"/Chooser/Model/Train/").mkdirs();
				new File(path+"/Chooser/Model/Test/").mkdirs();
				 writer_train = new BufferedWriter(new FileWriter(path+"/Chooser/Model/Train/pesos.dat",false));
		
				 writer_test = new BufferedWriter(new FileWriter(path+"/Chooser/Model/Test/pesos.dat",false));
				
				 autores = nombres[0] + "-" + nombres[1];
				//Cabecera del fichero
				 List<String> cabecera= new ArrayList<String>();
				 if(num_firma>0)cabecera.addAll(selector.getFirma(nombres));
				 if(num_function_words>0)cabecera.addAll(selector.getFunctionWords(nombres));
		
				//Escribo cabecera en ficheros train & test
				
				it2 = cabecera.iterator();
				
				String tag = it2.next().toString(); 
				writer_train.append('"'+tag+'"');
				writer_test.append('"'+tag+'"');
				while(it2.hasNext()){
					tag = it2.next().toString();
					writer_train.append("\t\""+tag+'"');
					writer_test.append("\t\""+tag+'"');
					
				}
				writer_train.append("\tSujeto\n");
				writer_test.append("\tSujeto\n");				
		
			} catch (Exception e) {					
				logger.severe(e.toString());
				return false;
			}
			
			pesosTrain.clear();
			pesosTest.clear();
			
			//En la variable nombres se encuentran los dos autores 
			//a los que se van a generar los modelos
			
			//Cargamos la lista de ficheros de entreno y de test y obtenemos los pesos
			
			for(String nombre:nombres){
				
				//Primero obtengo pesos de train
				directorio = new File(path+"/Documentos/"+nombre+"/results_Train");
				if(directorio.exists()){
					
					for(File doc:directorio.listFiles()){
						
						try {
							pesosTrain=selector.getCaracteristicas(doc, nombres);						
						
							it = pesosTrain.iterator();
							writer_train.append(it.next().toString());
							while(it.hasNext()){
								writer_train.append("\t"+it.next().toString());
							}
							writer_train.append("\t"+nombre+"\n");
							
							
						} catch (Exception e) {
							logger.severe(e.toString());
						}
						contador++;
					}
					if(contador==0){
						logger.severe("No se han encontrado fichero dentro del directorio: "+path+"/Documentos/"+nombre+"/results_Train");
						return false;
					}else{
						logger.info("Analizados "+contador+" ficheros de "+nombre+" para entreno de modelos");
						
					}
					contador = 0;
					//Obtengo pesos de test
					directorio = new File(path+"/Documentos/"+nombre+"/results_Test");
					if(directorio.exists()){
						
						for(File doc:directorio.listFiles()){
							
							try {
								pesosTest=selector.getCaracteristicas(doc, nombres);							
							
								it = pesosTest.iterator();
								writer_test.append(it.next().toString());
								while(it.hasNext()){
									writer_test.append("\t"+it.next().toString());
								}
								writer_test.append("\t"+nombre+"\n");
								
							} catch (Exception e) {
								logger.severe(e.toString());							
							}
							contador++;
						}
						if(contador==0){
							logger.severe("No se han encontrado fichero dentro del directorio: "+path+"/Documentos/"+nombre+"/results_Test");
							return false;
						}else{
							logger.info("Analizados "+contador+" ficheros de "+nombre+" para testeo de los modelos");

						}	
						contador = 0;
				}else{
					logger.severe("No se ha encontrado el directorio: "+path+"/Documentos/"+nombre+"/results_Train"+"No se pudieron cargar los documentos de entreno");
					return false;
				}
				
			}	
				
			
		}
		
			//Cierro flujos de datos
			try {
				writer_test.close();
				writer_train.close();
			} catch (IOException e) {
				logger.severe(e.toString());
			}
			//Creo los directorios de los modelos
		
			String pathChooser = path+"/Chooser/";
			 //Creo carpetas necesarias para cada modelo
			 
			 //DIRECTORIO RAIZ
			 directorio = new File(pathChooser+"Modelos/"+nombres[1]+"-"+nombres[0]);
				 if(directorio.exists()){
					directorio.delete();				
				 }
			 directorio = new File(pathChooser+"Modelos/"+nombres[0]+"-"+nombres[1]);
				 if(directorio.exists()){
						directorio.delete();
						directorio.mkdirs();
				 }else{
					 directorio.mkdirs();
				 }
			 
			 //DIRECTORIO SVM
			 directorio = new File(pathChooser+"Modelos/"+nombres[0]+"-"+nombres[1]+"/svm");
			 if(!directorio.exists()){
				 directorio.mkdirs();
			 }else{
				 directorio.delete();
				 directorio.mkdirs();
			 }
			 //DIRECORIO RED NEURONAL
			 directorio = new File(pathChooser+"Modelos/"+nombres[0]+"-"+nombres[1]+"/neuronal");
			 if(!directorio.exists()){
				 directorio.mkdirs();
			 }else{
				 directorio.delete();
				 directorio.mkdirs();
			 }
			 //DIRECTORIO RED BAYESIANA
			 directorio = new File(pathChooser+"Modelos/"+nombres[0]+"-"+nombres[1]+"/Bayes");
			 if(!directorio.exists()){
				 directorio.mkdirs();
			 }else{
				 directorio.delete();
				 directorio.mkdirs();
			 }
			 //DIRECTORIO VECTOR DE ERROR
			 directorio = new File(pathChooser+"Modelos/"+nombres[0]+"-"+nombres[1]+"/v_error");
			 if(!directorio.exists()){
				 directorio.mkdirs();
			 }else{
				 directorio.delete();
				 directorio.mkdirs();
			 }			 
			script.eval("k<-"+K); 
			script.eval("num_firma<-"+NUM_VARIABLES);
			script.eval("pca<-0");
			script.assign("name", autores);
			script.Reval(new File(path+"/ScriptR/modelv4.R"), "Chooser");
		
			System.out.println("Modelos creados correctamente");
		
		
		
		return true;
	}
	
	

	}
	
	public void terminate(){
		
		script.terminate();
	}
}
