package Utilities;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;




import alex.tfg.freeling.*;

public class CreateTagMatrix {
  // Modify this line to be your FreeLing installation directory
  private static final String FREELINGDIR = "/usr/local";
  private static final String DATA = FREELINGDIR + "/share/freeling/";
  private static final String LANG = "es";
  private final static Logger logger = Logger.getLogger(CreateTagMatrix.class.getName());
  //Directorio del sujeto
  private static File directorio = null; 
  
  private LangIdent lgid;
  private Tokenizer tk;
  private Splitter sp;
  private Maco mf;
  private HmmTagger tg;
  private ChartParser parser;
  private DepTxala dep;
  private Nec neclass;
  private Senses sen; 
  private Ukb dis;
  private StringBuffer tags;
  
  public CreateTagMatrix(){
	    System.loadLibrary( "freeling_javaAPI" ); 
	    Util.initLocale( "default" );

	    // Create options set for maco analyzer.
	    // Default values are Ok, except for data files.
	    MacoOptions op = new MacoOptions( LANG );

	    op.setActiveModules(false, true, true, true, 
	                               true, true, true, 
	                               true, true, true);

	    op.setDataFiles(
	      "", 
	      DATA+LANG+"/locucions.dat", 
	      DATA + LANG + "/quantities.dat",
	      DATA + LANG + "/afixos.dat",
	      DATA + LANG + "/probabilitats.dat",
	      DATA + LANG + "/dicc.src",
	      DATA + LANG + "/np.dat",
	      DATA + "common/punct.dat");

	    // Create analyzers.
	    lgid = new LangIdent(DATA + "/common/lang_ident/ident-few.dat");

	    tk = new Tokenizer( DATA + LANG + "/tokenizer.dat" );
	   sp = new Splitter( DATA + LANG + "/splitter.dat" );
	    mf = new Maco( op );

	   tg = new HmmTagger( DATA + LANG + "/tagger.dat", true, 2 );
	    parser = new ChartParser(
	     DATA + LANG + "/chunker/grammar-chunk.dat" );
	    dep = new DepTxala( DATA + LANG + "/dep/dependences.dat",
	     parser.getStartSymbol() );
	  neclass = new Nec( DATA + LANG + "/nerc/nec/nec-ab-poor1.dat" );

	   sen = new Senses(DATA + LANG + "/senses.dat" ); // sense dictionary
	 dis = new Ukb( DATA + LANG + "/ukb.dat" ); // sense disambiguator
	 
  }
  
  
  public StringBuffer createMatrix (File documento) throws IOException {
   
  /*  try{
    	LogManager.getLogManager().readConfiguration(new FileInputStream("./Properties/logger.properties"));
    
    }catch(Exception e){
    throw new RuntimeException("No se ha podido configurar el sistema de trazas: " +e.toString());
    }*/
    
    FileReader corpus = new FileReader(documento);
    tags = new StringBuffer("");    

   
    // Leemos uno a uno los ficheros.
    BufferedReader input = new BufferedReader(corpus);
    String line = input.readLine();
    ListWord l=null;
    ListSentence ls=null;
    while( line != null ) {
      
    if(!line.equals("")){	
    	// Extract the tokens from the line of text.
    	l = tk.tokenize( line );

      // Split the tokens into distinct sentences.
    	 ls = sp.split( l, false );
    	
      // Perform morphological analysis
   	try{
    	mf.analyze( ls );
   	}catch(Exception e){
   		System.out.println("Error:"+ e.toString());
   	}
      // Perform part-of-speech tagging.
      tg.analyze( ls );

      // Perform named entity (NE) classificiation.
      neclass.analyze( ls );

      sen.analyze( ls );
      dis.analyze( ls );
      

      // Chunk parser
      //parser.analyze( ls );
   //  printResults( ls, "parsed" ,i+1,name,false);

      // Dependency parser
      //dep.analyze( ls );
     // printResults( ls, "dep",i+1,name,false );
      printResults( ls, "tagged",false);
    }
      line = input.readLine();
    }
    
    input.close();    
    corpus.close();
    printResults(new ListSentence(),"",true);      
    
    return tags;
  }

  public void deleteAnalyzers(){
	  
	  lgid.delete();
	  tk.delete();
	  sp.delete();
	  mf.delete();
	  tg.delete();
	  parser.delete();
	  dep.delete();
	  neclass.delete();
	  sen.delete();
	  dis.delete();
	   
  }
  
  private  void printResults( ListSentence ls, String format,boolean finish)    {

	   
	   if(finish){
		 
			  tags.append("\n"); 
			  System.out.println( "-------- Documento Etiquetado-----------" );
	  }
	  else{
			 if(format=="tagged"){
		     
				  
			      // get the analyzed words out of ls.
			      ListSentenceIterator sIt = new ListSentenceIterator(ls);
			      Sentence s;
			      ListWordIterator wIt;
			      Word w;
				      while (sIt.hasNext()) {
				        s = sIt.next();
				        wIt = new ListWordIterator(s);
					    while (wIt.hasNext()) {
					        	w = wIt.next();				        
					        	tags.append(w.getTag()+" ");
						}	                
				      
				      }
		    	
		     }
		    
			
		    
	  }

  }
}




