package Utilities;

import java.io.File;
import java.util.logging.Logger;

import org.rosuda.JRI.REXP;
import org.rosuda.JRI.Rengine;



public class ExecuteScript {

	private final static Logger logger = Logger.getLogger(ExecuteScript.class.getName());
	
	
	private Rengine re;
	
	/**
	 * Inicializa el entorno R
	 */
	public ExecuteScript(){
		
		//Fuerzo al recolector de basura que libere memoria.
    	
		//Ahora mediante R empieza se crea la firma de la persona.
	
		 // just making sure we have the right version of everything
		if (!Rengine.versionCheck()) {
		   // logger.warning("** Version mismatch - Java files don't match library version.");
		    System.exit(1);
		}
	        //logger.info("Creating Rengine (with arguments)");
			// 1) we pass the arguments from the command line
			// 2) we won't use the main loop at first, we'll start it later
			//    (that's the "false" as second argument)
			// 3) the callbacks are implemented by the TextConsole class above

	        String [] argumentos = {"--no-save"};
	        re=new Rengine(argumentos, false, new textConsole());
	        logger.info("Rdocengine created, waiting for R");
			// the engine creates R is a new thread, so we should wait until it's ready
	        if (!re.waitForR()) {
	            logger.severe("Cannot load R");
	            System.exit(0);
	        }
	       logger.info("Entorno R iniciado correctamente");
	       //Entorno R iniciado	
		
		
	}
	
	/**
	 * Funcion Reval,
	 * Evalua el script pasado como parametro
	 * de tipo FILE
	 *  
	 * 
	 * @param Script Fichero del script a ejecutar
	 * @param name   Nombre a partir del directorio raiz del programa para establecer el are de trabajo del entorno
	 * @return
	 */
	public boolean Reval(File Script,String name){
		

	       
	       try{	    	   
	    	   if(re.isAlive()){
	    	  re.eval("setwd('"+Script.getParentFile().getParentFile().getAbsolutePath()+"/"+name+"')");
	    	 //Charged on memory the R-script
	    	  re.eval("source('"+Script.getAbsolutePath()+"')");
	    	   }else{
	    		   logger.severe("El entorno R no se encuentra activo");
	   			System.exit(0);
	    	   }
	       } catch (Exception e) {
			logger.severe("Error:"+e.toString());
				
	       }
	       
		 
		  
	       logger.info("Script:"+Script.getName() +"ejecutado correctamente");
		  
		  return true;
	}
	
	/**
	 * Asigna un valor a una variable
	 * @param name Nombre de la variable
	 * @param value valor de la variable
	 */
	public void assign(String name, String value){

		if(re.isAlive()){
			re.assign(name, value);	
		}else{
			logger.severe("El entorno R no se encuentra activo");
			System.exit(0);
		}
		
		
	}
	
	
	public String [] getStringArray(String name){
		
		REXP resultado = null;
		String names[]=null;
		if(re.isAlive()){
			resultado = re.eval(name);
			names = resultado.asStringArray();
			
		}else{
			logger.severe("El entorno R no se encuentra activo");
			System.exit(0);
		}
		return names;
	}
	
	public double [] getDoubleArray(String name){
		
		REXP resultado = null;
		double names[]=null;
		if(re.isAlive()){
			resultado = re.eval(name);
			names = resultado.asDoubleArray();
			
		}else{
			logger.severe("El entorno R no se encuentra activo");
			System.exit(0);
		}
		return names;
	}
	
	public void setDoubleArray(String name, double [] array) throws Exception{
		
		REXP resultado = null;

		if(re.isAlive()){
			if(!re.assign(name, array)) {
				logger.severe("No se pudo cargar en el entorno R el array: " + name);
				throw new Exception("No se pudo cargar el array de doubles");
			}
		}else{
			logger.severe("El entorno R no se encuentra activo");
			System.exit(0);
		}
	
	}
	
	/**
	 * Evalúa lo pasado por argumento
	 * 
	 * @param var
	 * @return
	 */
	public String eval(String var){
		
		String res=null;
		if(re.isAlive()){
			res = re.eval(var).asString();
		}else{
			logger.severe("El entorno R no se encuentra activo");
			System.exit(0);
		}
		return res;
	}
	
	public REXP evalVariable(String var){
		
		REXP res=null;
		if(re.isAlive()){
			res = re.eval(var);
		}else{
			logger.severe("El entorno R no se encuentra activo");
			System.exit(0);
		}
		return res;
	}
	
	public void terminate(){
		re.end();
	}
	
	public void limpiarConsola(){
		
		if(re.isAlive()){
			re.eval("rm(list=ls())");
		}else{
			logger.severe("El entorno R no se encuentra activo");
			System.exit(0);
		}
	}
	
}
