package Utilities;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.StringReader;
import java.util.Properties;
import java.util.logging.Logger;

/**
 * Clase que se encarga de gestionar
 * los tags
 * @author alejandro
 *
 */

public class TagsManager  {
	
	private final static Logger logger = Logger.getLogger(TagsManager.class.getName());
	
	//Variable que maneja la base de datos de TAGS 
	 private  Properties tags = new Properties();
	 //Directorio donde se encuentra el fichero de propiedades de los Tags
	 private static File path = new File("./Properties");
	 
	 public TagsManager(){
		
		 try {
				tags.load(new FileInputStream(path.getAbsolutePath()+"/tags.properties"));
			} catch (FileNotFoundException e) {
				logger.info("Error al leer el archivo de propiedades: Archivo tags.properties no encontrado");
				logger.severe(e.toString());
			} catch (IOException e) {
				logger.info("Error al cargar el archivo de propiedades");
				logger.severe(e.toString());
			}
	 }
	/**
	 * Funcion que devuelve el signficado de un Tag pasado como argumento.
	 * 
	 * @param Tag
	 * @return
	 */
	public String getMeaningTag(String Tag){
		
		StringReader reader = new StringReader(Tag);
		StringBuffer analisis = new StringBuffer();
		Tag=Tag.toUpperCase();
		//Significado de una parte de la etiqueta
		String etRes=null;
		try {
			char [] caracter = new char [1];
			int contador = 0;
			if(reader.read(caracter,0,1)!=-1){
				
				String etiqueta = new String();
				switch(caracter[0]){
				
				case 'A':
					analisis.append(tags.getProperty("A"));	
								
					while(reader.read(caracter,0,1)!=-1){
						
						switch(contador){
						
						//Tipo
						case 0:
							etiqueta = "AT";
							break;
						//Grado
						case 1:
							etiqueta = "AGR";
							break;
						//Genero
						case 2:
							etiqueta = "AG";
							break;
						//Numero
						case 3:
							etiqueta = "AN";
							break;
						//Funcion
						case 4:
							etiqueta = "AF";
							break;
						//Evitamos sorpresas	
						default:
							etiqueta = "Desconocido";
							break;							
						}
						
						if(!etiqueta.equals("Desconocido")){
							etRes= tags.getProperty(etiqueta+String.copyValueOf(caracter));
							if(!etRes.equals("No")){
							analisis.append(" "+etRes);
							}
						}
						else analisis.append(" propiedad desconocida");
						contador++;								
					}
					break;
				
				//Adverbio	
				case 'R':
					analisis.append(tags.getProperty("R"));	
								
					while(reader.read(caracter,0,1)!=-1){
						
						switch(contador){
						
						//Valor
						case 0:
							etiqueta = "RV";
							break;
					 //Evitamos sorpresas	
						default:
							etiqueta = "Desconocido";
							break;							
						}
						
						if(!etiqueta.equals("Desconocido")){
							etRes= tags.getProperty(etiqueta+String.copyValueOf(caracter));
							if(!etRes.equals("No")){
							analisis.append(" "+etRes);
							}
						}
						else analisis.append(" propiedad desconocida");	
						contador++;		
					}
					break;
				//Determinantes	
				case 'D':
					analisis.append(tags.getProperty("D"));	
								
					while(reader.read(caracter,0,1)!=-1){
						
						switch(contador){
						
						//Tipo
						case 0:
							etiqueta = "DT";
							break;
						//Persona
						case 1:
							etiqueta = "DP";
							break;
						//Genero
						case 2:
							etiqueta = "DG";
							break;
						//Numero
						case 3:
							etiqueta = "DN";
							break;
						//Poseedor
						case 4:
							etiqueta = "DP";
							break;
						//Evitamos sorpresas	
						default:
							etiqueta = "Desconocido";
							break;							
						}
						
						if(!etiqueta.equals("Desconocido")){
							etRes= tags.getProperty(etiqueta+String.copyValueOf(caracter));
							if(!etRes.equals("No")){
							analisis.append(" "+etRes);
							}
						}
						else analisis.append(" propiedad desconocida");
						contador++;							
					}
					break;
				//Nombres	
				case 'N':
					analisis.append(tags.getProperty("N"));	
								
					while(reader.read(caracter,0,1)!=-1){
						
						switch(contador){
						
						//Tipo
						case 0:
							etiqueta = "NT";
							break;
						//Genero
						case 1:
							etiqueta = "NG";
							break;
						//Numero
						case 2:
							etiqueta = "NN";
							break;
						//Clasificacion Semantica
						case 3:
							etiqueta = "NC";
							etiqueta = etiqueta+String.copyValueOf(caracter);
							reader.read(caracter,0,1);
							break;
						//grado
						case 4:
							etiqueta = "NG";														
							break;						
						//Evitamos sorpresas	
						default:
							etiqueta = "Desconocido";
							break;							
						}
						
						if(!etiqueta.equals("Desconocido")){
							etRes= tags.getProperty(etiqueta+String.copyValueOf(caracter));
							if(!etRes.equals("No")){
							analisis.append(" "+etRes);
							}
						}
						else analisis.append(" propiedad desconocida");
						contador++;								
					}
					break;
					
				//Verbos	
				case 'V':
					analisis.append(tags.getProperty("V"));	
								
					while(reader.read(caracter,0,1)!=-1){
						
						switch(contador){
						
						//Tipo
						case 0:
							etiqueta = "VT";
							break;
						//Modo
						case 1:
							etiqueta = "VM";
							break;
						//Tiempo
						case 2:
							etiqueta = "VT";
							break;
						//Persona
						case 3:
							etiqueta = "VP";
							break;
						//Numero
						case 4:
							etiqueta = "VN";
							break;
						//Genero
						case 5:
							etiqueta = "VG";
							break;
						//Evitamos sorpresas	
						default:
							etiqueta = "Desconocido";
							break;							
						}
						
						if(!etiqueta.equals("Desconocido")){
							etRes= tags.getProperty(etiqueta+String.copyValueOf(caracter));
							if(!etRes.equals("No")){
							analisis.append(" "+etRes);
							}
						}
						else analisis.append(" propiedad desconocida");
						contador++;		
					}
					break;
					//Pronombres	
					case 'P':
						analisis.append(tags.getProperty("P"));	
									
						while(reader.read(caracter,0,1)!=-1){
							
							switch(contador){
							
							//Tipo
							case 0:
								etiqueta = "PT";
								break;
							//Persona
							case 1:
								etiqueta = "PP";
								break;
							//Genero
							case 2:
								etiqueta = "PG";
								break;
							//Numero
							case 3:
								etiqueta = "PN";
								break;
							//Caso
							case 4:
								etiqueta = "PC";
								break;
							//Poseedor
							case 5:
								etiqueta = "PP";
								break;
							//Educacion
							case 6:
								etiqueta = "PE";
								break;
							//Evitamos sorpresas	
							default:
								etiqueta = "Desconocido";
								break;							
							}
							
							if(!etiqueta.equals("Desconocido")){
								etRes= tags.getProperty(etiqueta+String.copyValueOf(caracter));
								if(!etRes.equals("No")){
								analisis.append(" "+etRes);
								}
							}
							else analisis.append(" propiedad desconocida");
							contador++;							
						}
						break;
					//Conjunciones	
					case 'C':
						analisis.append(tags.getProperty("C"));	
									
						while(reader.read(caracter,0,1)!=-1){
							
							switch(contador){
							
							//Tipo
							case 0:
								etiqueta = "CT";
								break;
								//Evitamos sorpresas	
							default:
								etiqueta = "Desconocido";
								break;							
							}
							
							if(!etiqueta.equals("Desconocido")){
								etRes= tags.getProperty(etiqueta+String.copyValueOf(caracter));
								if(!etRes.equals("No")){
								analisis.append(" "+etRes);
								}
							}
							else analisis.append(" propiedad desconocida");
							contador++;								
						}
						break;
					//Interjecciones	
					case 'I':
						analisis.append(tags.getProperty("I"));	
									
						while(reader.read(caracter,0,1)!=-1){
							
							switch(contador){							
						
							default:
								etiqueta = "Desconocido";
								break;							
							}
							
							if(!etiqueta.equals("Desconocido")){
								etRes= tags.getProperty(etiqueta+String.copyValueOf(caracter));
								if(!etRes.equals("No")){
								analisis.append(" "+etRes);
								}
							}
							else analisis.append(" propiedad desconocida");
							contador++;						
						}
						break;
						
					//Preposiciones	
					case 'S':
						analisis.append(tags.getProperty("S"));	
									
						while(reader.read(caracter,0,1)!=-1){
							
							switch(contador){
							
							//Tipo
							case 0:
								etiqueta = "SP";
								break;
							//Forma
							case 1:
								etiqueta = "SF";
								break;
							//Genero
							case 2:
								etiqueta = "SG";
								break;
							//Numero
							case 3:
								etiqueta = "SN";
								break;
							//Evitamos sorpresas	
							default:
								etiqueta = "Desconocido";
								break;							
							}
							
							if(!etiqueta.equals("Desconocido")){
								etRes= tags.getProperty(etiqueta+String.copyValueOf(caracter));
								if(!etRes.equals("No")){
								analisis.append(" "+etRes);
								}
							}
							else analisis.append(" propiedad desconocida");
							contador++;						
						}
						break;
						
				//Signos de puntuacion
						case 'F':
							analisis.append(tags.getProperty("F"));	
							etRes = etiqueta+String.copyValueOf(caracter);
							while(reader.read(caracter,0,1)!=-1){
								
								etRes = etRes + String.copyValueOf(caracter);
							}
																					
							etRes= tags.getProperty(etRes);
							if(!etRes.equals("No")){
								analisis.append(" "+etRes);
							}
															
							break;
							
						//Cifras y numerales
						case 'Z':
							analisis.append(tags.getProperty("Z"));	
										
							while(reader.read(caracter,0,1)!=-1){
								
								switch(contador){
								
								//Tipo
								case 0:
									etiqueta = "ZT";
									break;
								//Evitamos sorpresas	
								default:
									etiqueta = "Desconocido";
									break;							
								}
								
								if(!etiqueta.equals("Desconocido")){
									etRes= tags.getProperty(etiqueta+String.copyValueOf(caracter));
									if(!etRes.equals("No")){
									analisis.append(" "+etRes);
									}
								}
								else analisis.append(" propiedad desconocida");
								contador++;						
							}
							break;
							
						//Decha y Hora
						case 'W':
							analisis.append(tags.getProperty("W"));	
							break;
				
				}
				return analisis.toString();
			}else{
				logger.severe("Ha introducido un Tag vacio");
				return "NOK";
			}
				
			
		} catch (IOException e) {
			
			logger.severe("Error al leer el caracter asociado al Tag "+ Tag);
			return "NOK";
		}

		
		
	}
	
	/**
	 * Funcion que devuelve el significado de un grupo de Tags
	 * @param group
	 * @return
	 */
	
	public String getGroupTag(String group){
		
		group=group.toUpperCase();
		String [] tags =  group.split(" ");
		StringBuffer resultado = new StringBuffer();
		
		for(int i=0;i<tags.length;i++){
			
			if(i<(tags.length-1))resultado.append(getMeaningTag(tags[i])+"+");
			else resultado.append(getMeaningTag(tags[i]));			
			
		}
		return resultado.toString();
	}
	
	

}
