package analizador;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import Utilities.RemoveEmptyLines;



import alex.tfg.freeling.*;

public class Analyzer {
  // Modify this line to be your FreeLing installation directory
  private static final String FREELINGDIR = "/usr/local";
  private static final String DATA = FREELINGDIR + "/share/freeling/";
  private static final String LANG = "es";
  private final static Logger logger = Logger.getLogger(Analyzer.class.getName());
  //Directorio del sujeto
  private static File directorio = null; 
  
  public static void main( String args[] ) throws IOException {
    System.loadLibrary( "freeling_javaAPI" );    
    try{
    	LogManager.getLogManager().readConfiguration(new FileInputStream("./Properties/logger.properties"));
    
    }catch(Exception e){
    throw new RuntimeException("No se ha podido configurar el sistema de trazas: " +e.toString());
    }
    System.out.println("ANALIZADOR DE DOCUMENTOS: MODULO 2");    
    //Lectura de documentos de Entreno
    String name=null;
    try{
     name = args[0];
    }catch(Exception e){
    	logger.severe("No ha introducido un nombre como argumento");
    	System.exit(0);
    }
    //Elimino la matriz de Tags en el caso de que ya halla alguna
    
    File dir_tags = new File("./Documentos/"+name+"/Tags/matrizTags.txt");
    if(dir_tags.exists()) if(dir_tags.delete()) logger.info("Matriz de Tags eliminada correctamente");
    else logger.severe("La matriz de Tags no se ha podido eliminar");
    
    System.out.println("Analisis documentos de carpeta Train del sujeto: "+name);
    try{
    	directorio = new  File("./Documentos/"+name+"/results_Train");
    }catch(Exception e){
    	logger.severe("No se ha podido encontrar la carpeta Train del sujeto: "+name);
    	System.exit(0);
    }
    List<FileReader> corpus = new ArrayList<FileReader>();
   
    for(File fichero: directorio.listFiles()){
    	
    	String path = fichero.getPath();
    	corpus.add(new FileReader(path));
    }


    Util.initLocale( "default" );

    // Create options set for maco analyzer.
    // Default values are Ok, except for data files.
    MacoOptions op = new MacoOptions( LANG );

    op.setActiveModules(false, true, true, true, 
                               true, true, true, 
                               true, true, true);

    op.setDataFiles(
      "", 
      DATA+LANG+"/locucions.dat", 
      DATA + LANG + "/quantities.dat",
      DATA + LANG + "/afixos.dat",
      DATA + LANG + "/probabilitats.dat",
      DATA + LANG + "/dicc.src",
      DATA + LANG + "/np.dat",
      DATA + "common/punct.dat");

    // Create analyzers.
    LangIdent lgid = new LangIdent(DATA + "/common/lang_ident/ident-few.dat");

    Tokenizer tk = new Tokenizer( DATA + LANG + "/tokenizer.dat" );
    Splitter sp = new Splitter( DATA + LANG + "/splitter.dat" );
    Maco mf = new Maco( op );

    HmmTagger tg = new HmmTagger( DATA + LANG + "/tagger.dat", true, 2 );
    ChartParser parser = new ChartParser(
      DATA + LANG + "/chunker/grammar-chunk.dat" );
    DepTxala dep = new DepTxala( DATA + LANG + "/dep/dependences.dat",
      parser.getStartSymbol() );
    Nec neclass = new Nec( DATA + LANG + "/nerc/nec/nec-ab-poor1.dat" );

    Senses sen = new Senses(DATA + LANG + "/senses.dat" ); // sense dictionary
    Ukb dis = new Ukb( DATA + LANG + "/ukb.dat" ); // sense disambiguator
   
    for(int i=0; i< corpus.size();i++){
    // Leemos uno a uno los ficheros.
    BufferedReader input = new BufferedReader(corpus.get(i));
    String line = input.readLine();
    
    while( line != null ) {
      
    if(!line.equals("")){	
    	// Extract the tokens from the line of text.
    	ListWord l = tk.tokenize( line );

      // Split the tokens into distinct sentences.
    	ListSentence ls = sp.split( l, false );

      // Perform morphological analysis
   	mf.analyze( ls );

      // Perform part-of-speech tagging.
      tg.analyze( ls );

      // Perform named entity (NE) classificiation.
      neclass.analyze( ls );

      sen.analyze( ls );
      dis.analyze( ls );
      

      // Chunk parser
      parser.analyze( ls );
   //  printResults( ls, "parsed" ,i+1,name,false);

      // Dependency parser
      dep.analyze( ls );
     // printResults( ls, "dep",i+1,name,false );
      printResults( ls, "tagged",i+1,name,false);
    }
      line = input.readLine();
    }
    
    input.close();
    printResults(new ListSentence(),"",0,name,true);
    
    }
    
    System.out.println("FIN MODULO 2");
  }

  private static void printSenses( Word w ) {
    String ss = w.getSensesString();

    // The senses for a FreeLing word are a list of
    // pair<string,double> (sense and page rank). From java, we
    // have to get them as a string with format
    // sense:rank/sense:rank/sense:rank
    // which will have to be splitted to obtain the info.
    //
    // Here, we just output it:
    System.out.print( " " + ss );
  }
  
  
  private static void printResults( ListSentence ls, String format,int id,String name,boolean finish)    {

    File directorio = new File("./Documentos/"+name+"/Tags");
	  if(!directorio.exists()){
		  directorio.mkdir();
	  }
	  FileWriter matrizTags=null;
	try{  
	//The true statement let to append data to the file.
	matrizTags = new FileWriter(directorio.getPath()+"/matrizTags.txt",true); 
	}catch(Exception e){
		logger.severe("Problemas al escribir en la Matriz de Tags: "+e.toString());
		System.exit(0);
	}
	 BufferedWriter writer = new BufferedWriter(matrizTags);
	  if(finish){
		  try{
			  writer.write("\n");  
			 
			  }catch(Exception e){
				  logger.severe("Error al terminar documento en matriz de Tags" + e.toString());
				  System.exit(0);
			  }  
	  }
	  else{
			 if (format == "parsed") {
		      System.out.println( "-------- CHUNKER results -----------" );
		
		      ListSentenceIterator sIt = new ListSentenceIterator(ls);
		      while (sIt.hasNext()) {
			Sentence s = sIt.next();
		        TreeNode tree = s.getParseTree();
		        printParseTree( 0, tree );
		      }
		    }
		    else if (format == "dep") {
		      System.out.println( "-------- DEPENDENCY PARSER results -----------" );
		
		      ListSentenceIterator sIt = new ListSentenceIterator(ls);
		      while (sIt.hasNext()) {
			Sentence s = sIt.next();
		        TreeDepnode tree = s.getDepTree();
		        printDepTree( 0, tree);
		      }
		    }
		    else
		    {
		      System.out.println( "-------- Documento"+id+"Etiquetado-----------" );
		
		      // get the analyzed words out of ls.
		      ListSentenceIterator sIt = new ListSentenceIterator(ls);
		      while (sIt.hasNext()) {
		        Sentence s = sIt.next();
		        ListWordIterator wIt = new ListWordIterator(s);
		        while (wIt.hasNext()) {
		          Word w = wIt.next();
				        try{
				         writer.write(w.getTag()+"\t");
				        }catch(Exception e){
				        	logger.severe("Error al escribir en la matriz de Tags: " + e.toString());
				        }	                
		          
		        }
		    
		     }
		        
		    }
	  }
	  try{
	  writer.close();
	  }catch(Exception e){
		logger.severe("No se ha podido cerra la matriz de Tags: "+e.toString());  
	  }
}

  private static void printParseTree( int depth, TreeNode tr ) {
    Word w;
    TreeNode child;
    long nch;

    // Indentation
    for( int i = 0; i < depth; i++ ) {
      System.out.print( "  " );
    }

    nch = tr.numChildren();

    if( nch == 0 ) {
      // The node represents a leaf
      if( tr.getInfo().isHead() ) {
        System.out.print( "+" );
      }
      w = tr.getInfo().getWord();
      System.out.print(
        "(" + w.getForm() + " " + w.getLemma() + " " + w.getTag() );
      printSenses( w );
      System.out.println( ")" );
    }
    else
    {
      // The node probably represents a tree
      if( tr.getInfo().isHead() ) {
        System.out.print( "+" );
      }

      System.out.println( tr.getInfo().getLabel() + "_[" );

      for( int i = 0; i < nch; i++ ) {
        child = tr.nthChildRef( i );

        if( child != null ) {
          printParseTree( depth + 1, child );
        }
        else {
          System.err.println( "ERROR: Unexpected NULL child." );
        }
      }
      for( int i = 0; i < depth; i++ ) {
        System.out.print( "  " );
      }

      System.out.println( "]" );
    }
  }

  private static void printDepTree( int depth, TreeDepnode tr ) {
    TreeDepnode child = null;
    TreeDepnode fchild = null;
    Depnode childnode;
    long nch;
    int last, min;
    Boolean trob;

    for( int i = 0; i < depth; i++ ) {
      System.out.print( "  " );
    }

    System.out.print(
      tr.getInfo().getLinkRef().getInfo().getLabel() + "/" +
      tr.getInfo().getLabel() + "/" );

    Word w = tr.getInfo().getWord();

    System.out.print(
      "(" + w.getForm() + " " + w.getLemma() + " " + w.getTag() );
    printSenses( w );
    System.out.print( ")" );

    nch = tr.numChildren();

    if( nch > 0 ) {
      System.out.println( " [" );

      for( int i = 0; i < nch; i++ ) {
        child = tr.nthChildRef( i );

        if( child != null ) {
          if( !child.getInfo().isChunk() ) {
            printDepTree( depth + 1, child );
          }
        }
        else {
          System.err.println( "ERROR: Unexpected NULL child." );
        }
      }

      // Print chunks (in order)
      last = 0;
      trob = true;

      // While an unprinted chunk is found, look for the one with lower
      // chunk_ord value.
      while( trob ) {
        trob = false;
        min = 9999;

        for( int i = 0; i < nch; i++ ) {
          child = tr.nthChildRef( i );
          childnode = child.getInfo();

          if( childnode.isChunk() ) {
            if( (childnode.getChunkOrd() > last) &&
                (childnode.getChunkOrd() < min) ) {
              min = childnode.getChunkOrd();
              fchild = child;
              trob = true;
            }
          }
        }
        if( trob && (child != null) ) {
          printDepTree( depth + 1, fchild );
        }

        last = min;
      }

      for( int i = 0; i < depth; i++ ) {
        System.out.print( "  " );
      }

      System.out.print( "]" );
    }

    System.out.println( "" );
  }
}

