package chooser;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Vector;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import org.rosuda.JRI.REXP;
import org.rosuda.JRI.RFactor;
import org.rosuda.JRI.RVector;

import selector.caracteristicas.Selector;

import beans.Autor;
import beans.Documento;
import beans.prediccion;

import Utilities.ExecuteScript;
import Utilities.findTagSign;


/**
 * Clase que se encarga de
 * elegir un autor entre varios
 * posibles
 * @author alejandro
 *
 */
public class Chooser {

	private final static Logger logger = Logger.getLogger(Chooser.class.getName());
	public static void main( String Args[] ) throws IOException {
		
		 try{
		    	LogManager.getLogManager().readConfiguration(new FileInputStream("./Properties/logger.properties"));
		    
		    }catch(Exception e){
		    throw new RuntimeException("No se ha podido configurar el sistema de trazas: " +e.toString());
		    }
		 
			System.out.println("MODULO DE DECISON DE AUTORIA ENTRE SUJETOS");
			
			
			//Cargo propiedades de la plataforma
		
			//Leyendo fichero de configuracion de la plataforma
			 System.out.println("Cargando fichero de configuracion del extractor..."); 
			//Variable que lee del fichero de propiedades
			 Properties config = new Properties();
			 //Directorio donde se encuentran los ficheros de propiedades
			 String path_prop = new File("./Properties").getAbsolutePath();
			 
			 try {
					config.load(new FileInputStream(path_prop+"/nucleo.properties"));
				} catch (FileNotFoundException e) {
					logger.info("Error al leer el archivo de propiedades: Archivo nucleo.properties no encontrado");
					logger.severe(e.toString());
					System.exit(0);
				} catch (IOException e) {
					logger.info("Error al cargar el archivo de propiedades nucleo.properties");
					logger.severe(e.toString());
					System.exit(0);
				}
			 
			 System.out.println("Fichero de propiedades cargado correctamente");
			 String num_firma =config.getProperty("num_firma");
			 String num_func_words =config.getProperty("num_func_words");
			 String PCA = config.getProperty("PCA");
			
			//Seleciono cuales son los nombres entre los que voy a decidir la autoria
			 String [] names = Args;
			 
			 //Inicio entorno R para ejecutar scripts
			 ExecuteScript script = new ExecuteScript();
			 
			 
			//Cargo el/los documento/s de la carpeta pick

			 //Creo lista docs que seran analizados
			List<Documento> ListaDocumentos = new ArrayList<Documento>();
 
			String path = new File("").getAbsolutePath();
			File documento = new File(path+"/Chooser/pick");
			if(documento.exists()){
				
				try	{
				File documentos[] = documento.listFiles();	
				if(documentos.length > 0){
					
					//Creo si no esta creado el directorio Analyze
					//En analyze se almacena todo la informacion de analisis(Pesos de cada una de las firmas)
					File directorio = new File(path+"/Chooser/Analyze");
					if(!directorio.exists())directorio.mkdirs();
					BufferedWriter writer=null;//Escribe pesos del doc en fichero pesos.dat
					Selector selector= new Selector(Integer.parseInt(num_firma),Integer.parseInt(num_func_words));//Obtener pesos del doc
					String sujetos = null;//Almaceno el nombre de la pareja de sujetos
					
					for(File texto:documentos){	
						
						System.out.println("\n\nDECISOR FICHERO: "+ texto.getName());
						script.eval("vector.nombres<- data.frame()");
						for(int i=0;i<names.length;i++){
							
							writer=null;
							
							for(int j=i+1;j<names.length;j++){
								
								sujetos = names[i]+"-"+names[j];
								String [] nombres = {names[i],names[j]};
								directorio = new File(path+"/Chooser/Analyze/"+sujetos);
								if(!directorio.exists())directorio.mkdir();
								directorio = new File(path+"/Chooser/Analyze/"+sujetos+"/pesos.dat");
														
								//Obtengo la firma de la pareja actual de sujetos
								System.out.println("Cargando la firma de los sujetos:" +sujetos);
								List<String> cabecera = new ArrayList<String>();
								//Cabecera del fichero
								if(Integer.parseInt(num_firma)>0)cabecera.addAll(selector.getFirma(nombres));
								if(Integer.parseInt(num_func_words)>0)cabecera.addAll(selector.getFunctionWords(nombres));
								
								System.out.println("Obtengo los pesos del documento asociados a la firma: "+sujetos);
								//Obtengo iterador de la firma
								Iterator it =cabecera.iterator();
								
								//Comienzo a escribir en el fichero
								writer = new BufferedWriter(new FileWriter(directorio,false));
								//Introduzco la cabecera del fichero (Los TAGS de la firma)
								writer.write('"'+it.next().toString()+'"');
								while(it.hasNext()){
								
									writer.append("\t\""+it.next().toString()+'"');
								}
								writer.append("\n");
								writer.flush();
								
								//Obtengo los pesos del documento asociados a la firma
								List<Float> doc = selector.getCaracteristicas(texto, nombres);
								//Obtengo el iterador de los pesos
								it = doc.iterator();
								
								//Concateno al fichero anterior los pesos asociados a la firma(CABECERA)
								writer.append(it.next().toString());
								while(it.hasNext()){
									writer.append("\t"+it.next().toString());
								}
								writer.append("\n");
								
								writer.flush();
								//Cierro el fichero 
								writer.close();
								
								System.out.println("Pesos del fichero:"+texto.getName()+", asociados a la firma:"+sujetos+" guardados correctamente");								
								
								System.out.println("Inicio proceso de decision del fichero: "+texto.getName()+" con el modelo: "+sujetos);	
								
								/*
								 * Comprobamos la existencia del modelo
								 * de la pareja de autores
								 */
								
								if(!new File(path+"/Chooser/Modelos/"+nombres[0]+"-"+nombres[1]).exists()){
								
									if(!new File(path+"/Chooser/Modelos/"+nombres[1]+"-"+nombres[0]).exists()){
										logger.severe("No hay modelo creado para los sujetos: "+nombres[0]+" y" +nombres[1]);
									}else{
										script.assign("dir.modelos","Modelos/"+nombres[1]+"-"+nombres[0]);
										
									}
										
								}else{
									script.assign("dir.modelos","Modelos/"+nombres[0]+"-"+nombres[1]);
									
								}
								
								if(!new File(path+"/Chooser/Analyze/"+nombres[0]+"-"+nombres[1]).exists()){
									
									if(!new File(path+"/Chooser/Analyze/"+nombres[1]+"-"+nombres[0]).exists()){
										logger.severe("No hay pesos creados en la carpeta analyze para los sujetos: "+nombres[0]+" y" +nombres[1]);
									}else{
										sujetos = nombres[1] + "-" + nombres[0];
									}
									
								}else{
									
									sujetos = nombres[0] + "-" + nombres[1];
								}
								script.assign("nombre.sujetos",sujetos);
								script.assign("dir.pesos","Analyze/"+sujetos);
								
								
								if(PCA.equals("SI")){
									if(!new File(path+"/Chooser/PCA/"+nombres[0]+"-"+nombres[1]).exists()){
										
										if(!new File(path+"/Chooser/PCA/"+nombres[1]+"-"+nombres[0]).exists()){
											logger.severe("No hay vector PCA creado en la carpeta analyze para los sujetos: "+nombres[0]+" y" +nombres[1]);
										}else{
											script.assign("dir.pca","PCA/"+nombres[1]+"-"+nombres[0]);
										}
										
									}else{
										
										script.assign("dir.pca","PCA/"+nombres[0]+"-"+nombres[1]);
									}	
									script.eval("pca<-1");
								}else{
									script.eval("pca<-0");
								}
								
								script.Reval(new File("./ScriptR/pickerv2.R"), "Chooser");
								System.out.println("\n\n");
																
							}
						}
							System.out.println("Resultado para el documento: "+texto.getName()+":");
							
							script.eval("print(vector.nombres)");
							REXP exp = script.evalVariable("vector.nombres");
							RVector vectorNombres = exp.asVector();
							RFactor resultado = vectorNombres.at(0).asFactor();
							int contador=0;
							String prediccion=null;
							Vector lista = new Vector();
							List<prediccion>  predicciones = new ArrayList<prediccion> ();
							for(int a=0;a<names.length;a++){
								
								
								for(int j=a+1;j<names.length;j++){
									sujetos = names[a]+"-"+names[j];
									if(names[a].equals(resultado.at(contador++))){
										predicciones.add(new prediccion(sujetos,names[a],names[j]));		
									}else{
										predicciones.add(new prediccion(sujetos,names[j],names[a]));		
									}
															
								}
								
							}
							
							//En la lista de predicciones estan almacenada todas las prediccione asociadas a cada modelo.
							
							Iterator<prediccion> it = predicciones.iterator();
							prediccion pre =null;
							List<String> posSujetos = new ArrayList<String>();
							List<String> negSujetos = new ArrayList<String>();
							Iterator<String> posIt=null;
							Iterator<String> negIt=null;
							boolean encontrado=false;
	 						while(it.hasNext()){				
	 							pre = it.next();
	 							encontrado=false;
		 						if(posSujetos.size()==0 && negSujetos.size()==0){
		 							
		 							posSujetos.add(pre.getPrediccion());
		 							negSujetos.add(pre.getNoPrediccion()); 	
		 							
		 						}else{
		 							posIt = posSujetos.iterator();
		 	 						negIt = negSujetos.iterator();
		 	 						
		 	 						while(posIt.hasNext()){
		 	 							String posible = posIt.next();
		 	 							if(pre.getNoPrediccion().equals(posible)){
		 	 								posIt.remove(); 	
		 	 								encontrado = true;
		 	 								break;
		 	 							}
		 	 							if(pre.getPrediccion().equals(posible)) encontrado = true;
		 	 						}
		 	 						if(!encontrado) posSujetos.add(pre.getPrediccion());
		 	 						encontrado=false;
		 	 						
		 	 						while(negIt.hasNext()){
		 	 							if(pre.getNoPrediccion().equals(negIt.next())){
		 	 								encontrado= true;
		 	 								break;
		 	 							}
		 	 						}
		 	 						if(!encontrado)negSujetos.add(pre.getNoPrediccion()); 
		 	 						
		 	 						//Actualizo lista posibles sujetos
		 	 						
		 	 						posIt = posSujetos.iterator();
		 	 					while(posIt.hasNext()){
		 	 						negIt = negSujetos.iterator();
		 	 						String posible = posIt.next();
		 	 						while(negIt.hasNext()){
		 	 							if(negIt.next().equals(posible)) posIt.remove();
		 	 						}
		 	 					}
		 	 						
		 	 							
		 						}					
									
							}
						
							
						if(posSujetos.size()==1){
							ListaDocumentos.add(new Documento(texto.getName(),posSujetos.get(0)));					
							
						}else{
							ListaDocumentos.add(new Documento(texto.getName(),"Desconocido"));
						}
							
							
							
						
					}	
					
					System.out.println("Ya se han analizado todos los documentos.");
					System.out.println("\n>--------------------<");
					System.out.println("RESULTADOS:");
					System.out.println(">--------------------<");
					Documento doc=null;
					
					List<Autor> autores = new ArrayList<Autor>();
					for(String autor_nombre:names){
						autores.add(new Autor(autor_nombre));
					}
					
					for(int i=0;i<ListaDocumentos.size();i++){						
						doc = ListaDocumentos.get(i);
						System.out.println((i+1)+"-     Nombre:"+doc.getNombre()+"     Autor:"+doc.getAutor());
			
						String autor_pred [] = doc.getAutor().split("_");
						StringBuffer autorLetras = new StringBuffer();
						for(String nom:autor_pred){
							autorLetras.append(nom.charAt(0));
						}
						if(doc.getNombre().startsWith(autorLetras.toString())){
							
							for(Autor autor:autores){
								if(autor.getNombre().equals(doc.getAutor())){
									autor.setAcertados(autor.getAcertados()+1);
								}
							}
							
						}else{
							for(Autor autor:autores){
								if(autor.getNombre().equals(doc.getAutor())){
									autor.setFallados(autor.getFallados()+1);
								}
							}
							
						}
					}
					System.out.println("Tabla de predicciones:");
					System.out.println("Autor|\t|Aciertos|\t|Fallos|\t|Total|");
					int total = 0;
					double acierto;
					for(Autor autor:autores){
					
					System.out.println(autor.getNombre()+"|\t|"+autor.getAcertados()+"|\t|"+autor.getFallados()+"|\t|"+(autor.getAcertados()+autor.getFallados()));
					System.out.println("------------------------------------------------");
					total += autor.getAcertados();
					
					}
					System.out.println(">>Estadisticas<<");
					System.out.println(total+" documentos acertados de "+ ListaDocumentos.size() );
					acierto =  (((double)total/(double)ListaDocumentos.size())*100);
					System.out.println("Porcentaje de acierto: "+ acierto);
				}else{
					System.out.println("No existe la carpeta donde se almacenan los documentos");
					System.exit(0);
				}			
			}catch(Exception e){
					System.out.println("Error con los ficheros: "+e.toString());
					logger.severe(e.toString());
				}
			}else{
				System.out.println("No existe la carpeta donde se almacenan los documentos del sujeto: ");
				System.exit(0);
			}
			script.terminate();
	}
}
