package extractor.caracteristicas;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import selector.caracteristicas.FunctionWordsExtractor;
import selector.caracteristicas.PesosFirmaExtractor;
import selector.caracteristicas.Selector;

import Utilities.ExecuteScript;


/**
 * Clase AnalisisPCA
 * 
 * Obtiene a partir de los documentos de
 * entreno vectores que relacionan las 
 * caracterísitcas para formar los componentes
 * principales
 * @author alejandro
 *
 */
public class AnalisisPCA {
	
	//Variable que sirve como sistema de trazas
	static Logger logger = Logger.getLogger(AnalisisPCA.class.getName());
	
	//Variable para ejecutar scrips en R
	private ExecuteScript script;
	  
	//Variable que contiene todas las caracteristicas
	private List<Float> caracteristicas = new ArrayList();
		
	//Variable que obtiene las pesos de las palabras funcionales
	Selector selector;
	
	
	  
	 /**
	  * Constructor de la clase AnalisisPCA,
	  * 
	  * inicializa el entorno R  
	  */
	 public AnalisisPCA(ExecuteScript script){
		 
		 this.script = script;
		 
	 }
	 
	 /**
	  * Funcion que obtiene los Componentes principales de
	  * dos personas
	  * 
	  * @param nombres Nombres de los autores
	  * @return True si se han obtenido correctamente los componentes principales. 
	  */
	 public boolean getPrincipalComponents(String numPCA,String numFirma,String numFuncWords,String tamFirma,String ... nombres){
		 
		 
		 	System.out.println("Preparando selector de caracteristicas...");
		 	this.selector = new Selector(Integer.parseInt(numFirma),Integer.parseInt(numFuncWords));
		 	System.out.println("Selector cargado correctamente");
			String path = new File("").getAbsolutePath();
			File directorio;
			List<Float> pesosTrain = new ArrayList();
			BufferedWriter writer_train = null;
			
			String autores;
			//Si ha introducido menos de dos nombres no se generan los modelos
			if(nombres.length<2){
				logger.severe("Ha introducido menos de dos autores para generar modelos");
				return false;
			}else{
				
				
				System.out.println("INICIO PROCESO DE OBTENCION DE COMPONENTES PRINCIPALES");
				
				//Creo cabecera de ficheros de pesos
				try {
					new File(path+"/Chooser/Model/Train/").mkdirs();//Si no esta creado el directorio se crea
					
					//Fichero pesos.dat para análisis PCA en entorno R
					writer_train = new BufferedWriter(new FileWriter(path+"/Chooser/Model/Train/pesos.dat",false));
				
					autores = nombres[0] + "-" + nombres[1];
					List<String> cabecera = new ArrayList<String>();
					//Cabecera del fichero
					if(Integer.parseInt(numFirma)>0)cabecera.addAll(selector.getFirma(nombres));
					if(Integer.parseInt(numFuncWords)>0)cabecera.addAll(selector.getFunctionWords(nombres));
					
					//Guardo la cabecera en el fichero pesos.dat
					Iterator<String> it = cabecera.iterator();
					
					String tag = it.next().toString(); 
					writer_train.append('"'+tag+'"');					
					while(it.hasNext()){
						tag = it.next().toString();
						writer_train.append("\t\""+tag+'"');					
						
					}
					writer_train.append("\tSujeto\n");
					
					pesosTrain.clear();
					
					//En la variable nombres se encuentran los dos autores 
					//de los cuales vamos a obtener las componentes principales
					
					//Cargamos la lista de ficheros de entreno y de test y obtenemos los pesos
					int contador;
					for(String nombre:nombres){
						
						contador = 0;
						//Primero obtengo pesos de train
						directorio = new File(path+"/Documentos/"+nombre+"/results_Train");
						if(directorio.exists()){
							
							for(File doc:directorio.listFiles()){
								
								try {
									pesosTrain=selector.getCaracteristicas(doc, nombres);						
								
									Iterator<Float> it2 = pesosTrain.iterator();
									writer_train.append(it2.next().toString());
									while(it2.hasNext()){
										writer_train.append("\t"+it2.next().toString());
									}
									writer_train.append("\t"+nombre+"\n");
									
									
								} catch (Exception e) {
									logger.severe(e.toString());
									script.terminate();
									return false;
								}
								contador++;
							}
							if(contador==0){
								logger.severe("No se han encontrado documentos dentro del directorio: "+path+"/Documentos/"+nombre+"/results_Train");
								return false;
							}else{
								logger.info("Analizados "+contador+" ficheros de "+nombre+" para entreno de modelos");								
							}
						}
					}					
				
				}catch (Exception e) {	
					try {
						writer_train.close();
					} catch (IOException e1) {
						// TODO Auto-generated catch block
						logger.severe(e1.toString());
						script.terminate();
						return false;
					}
					logger.severe(e.toString());
					return false;
				}
				//Se cierra fichero donde se almacenan los pesos de los docs
				//para realizar el analisis PCA
				try {
					writer_train.close();
				} catch (IOException e1) {
					// TODO Auto-generated catch block
					logger.severe(e1.toString());
					script.terminate();
					return false;
				}
				//Una vez obtenidos los pesos de los documentos de train se hace el analisis PCA en el entorno R
				
				//Lo primero es crear carpetas para almacenar vector eigen
				
				new File(path+"/Chooser/PCA/"+nombres[0]+"-"+nombres[1]).mkdirs();
							
				try {
					//Cargo el numero de firmas
				  	 script.eval("num_firma<-"+tamFirma);
				  	//Cargo el numero de componenetes principales
				  	 script.eval("num_PCA<-"+numPCA);				  	 
				  	//Cargo los nombres
				  	 script.assign("name",nombres[0]+"-"+nombres[1]);
					 script.Reval(new File(path+"/ScriptR/PCA.R"), "Chooser");
						
					writer_train.close();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					logger.severe(e.toString());
					script.terminate();
				}
				return true; //Se obtuvieron los componentes principales de forma correcta	
			}
		 
		 
		 
	 }
	

}
