package extractor.caracteristicas;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.logging.Logger;


import Utilities.ExecuteScript;
import Utilities.TagsManager;


/**
 *  Clase CreateSign,
 *  
 *  Clase que se encarga de todo lo
 *  relativo a la creación de la firma
 *  de uno o varios autores
 * 
 * @author alejandro
 *
 */
public class CreateSign {
	
	//Variable que sirve como sistema de trazas
	static Logger logger = Logger.getLogger(CreateSign.class.getName());
	
	 //Variable para ejecutar scrips en R
	  private ExecuteScript script;
	   //Variable que traduce la firma a texto entendible
	  private TagsManager tagsManager = new TagsManager();
	
	  /**
	   * Constructor de la clase CreateSign
	   * Toma de entrada la clase que
	   * se encarga de ejecutar codigo en R
	   * 
	   * @param script
	   */
	  public CreateSign(ExecuteScript script){
		  this.script = script;
	  }
	  
	  /**
	   * Función createSign,
	   * Permite obtener la firma identificativa 
	   * de el/los autor/res 
	   * 
	   * @param numWords Numero de tags en la firma
	   * @param nombres Nombres de los autores que conforman la firma
	   * @return
	   */
	  public boolean createSign(String numWords,String num_outliers_max,String range_valor_whiskers,String box_plot,String token_min,String token_max,String...names){
		  
				  
		  String line;
		  BufferedReader reader=null; 		
		  BufferedWriter writer = null;		 
		  File fichero =null;
		  String nombre_fichero = null;
		  String path = new File("").getAbsolutePath();
		 
		  String pathChooser = path+"/Chooser";		 
		  String pathDocumentos = path+"/Documentos";		  
		  String pathScripts = path+"/ScriptR";
		  

		  
		  if(names.length==2){
				  
				//Vaciamos entorno R
				script.limpiarConsola();
				
				//Compruebo que la carpeta Tags existe
				fichero = new File(pathChooser+"/Tags");
				fichero.mkdir();				
				//Guardamos la matriz de Tags con los dos sujetos en el directorio donde se procederá a crear el modelo
				try {
					nombre_fichero = "matrizTags_"+names[0]+"_"+names[1]+".dat";
					fichero = new File(pathChooser+"/Tags/"+nombre_fichero);						
					writer = new BufferedWriter(new FileWriter(fichero));		

		   		} catch (IOException e) {
					logger.severe("Problemas al crear la matriz de Tags: "+nombre_fichero);
					logger.info(e.toString());
					return false;
				}	
			  
				System.out.println("Creando firma para los sujetos: "+names[0]+" y "+names[1]);
				  //Junto las matrices de cada una de las parejas de los Sujetos
				
				
				try {
					reader = new BufferedReader(new FileReader(pathDocumentos+"/"+names[0]+"/Tags/matrizTags.dat"));
				
						try {
							line = reader.readLine();
							while(line!=null){	
								writer.write(line);								
								line = reader.readLine();
								writer.write("\n");
							}
							//Añado una coma para diferenciar distintos autores
							writer.write(",\n");
						}catch (IOException e) {
							
							logger.severe("Ha ocurrido un problema al leer la matriz de Tags de "+ names[0]);
							logger.info(e.toString());
							return false;
							
						}
					try {
						reader.close();
					} catch (IOException e1) {
						// TODO Auto-generated catch block
						logger.severe("Ha ocurrido un problema al leer de la matriz de Tags del sujeto: "+names[0]);
						logger.info(e1.toString());
					}
				
					reader = new BufferedReader(new FileReader(pathDocumentos+"/"+names[1]+"/Tags/matrizTags.dat"));					
						try {
								line = reader.readLine();
								while(line!=null){	
									writer.write(line);								
									line = reader.readLine();
									if(line!=null)writer.write("\n");
								}							
						} catch (IOException e) {
							
							logger.severe("Ha ocurrido un problema al leer la matriz de Tags del sujeto: "+names[1]);
							logger.info(e.toString());
							return false;
						}
					
							
				 } catch (FileNotFoundException e) {
						
					logger.severe("No existe el fichero matrizTags.dat en alguno de los sujetos");
					logger.info(e.toString());
					return false;
				}	
				//Cierro el fichero de escritura
				try {
					writer.close();
				} catch (IOException e2) {
					logger.severe(e2.toString());	
					return false;
				}
			   	
				//Compruebo que el directorio para la fimra exista,sino es así, lo creo.
		   		File dir_tags = new File(pathChooser+"/Firma/"+names[0]+"-"+names[1]);
		   		if(!dir_tags.exists()){
		   			dir_tags.mkdirs();
		   		}	
		   		//Ejecuto el script que calcula la firma de los sujetos
				script.assign("name",names[0]+"-"+names[1]);
				script.assign("fich.tag.matriz",nombre_fichero);
				//Cargo el numero de tags
			  	 script.eval("num_firma<-"+numWords);
			  	//Cargo el numero máximo de outliers permitidos
			  	 script.eval("num_outliers_max<-"+num_outliers_max);
				//Cargo valor de los bigotes del boxplot
			  	 script.eval("range.valor<-"+range_valor_whiskers);
			  	//Cargo variable para imprimir por pantalla resultado del  box-plot
			  	 script.eval("plot.estado<-"+box_plot);
			  	//Cargo el numero minimo de tokens en cada tag
			  	 script.eval("token.min<-"+token_min);
			 	//Cargo el numero maximo de tokens en cada tag
				 script.eval("token.max<-"+token_max);
				 
			  	script.Reval(new File(pathScripts+"/Ngra6-outliers-new.R"), "Chooser");
				
			 			   	
				//Una vez obtenida la firma en forma de "tags"
			   	//Se traducen esos tags a grupos de palabras "comprensibles"
			   	//Guardo en un fichero el significado de cada Tag
			   	String [] tags = script.getStringArray("tags");
			   	if(tags==null){
			   		
			   		logger.severe("No se pudo obtener la lista de tags");
			   		return false;
			   	}
			   	try {
					writer = new BufferedWriter(new FileWriter(new File(path+"/Chooser/Firma/"+names[0]+"-"+names[1]+"/firmaTexto.dat")));
				} catch (IOException e1) {
					logger.severe("No se pudo crear el fichero de la firma traducida");
					return false;
				}
				for(String tag:tags){
					try {
						writer.append(tagsManager.getGroupTag(tag)+"\n");
					} catch (IOException e) {
						logger.severe("No se pudo guardar en el fichero: "+path+"/Chooser/Firma/"+names[0]+"-"+names[1]+"/firmaTexto.dat"+"el tag:"+tag );
						return false;
					}
				}
			    try {
					writer.close();
					reader.close();
					
				
				//  Elimino el fichero matrizTags.dat para nuevo uso
				//if(!fichero.delete())logger.severe("No se ha podido eliminar la matriz de Tags");
				
				} catch (IOException e) {
					logger.severe("Ha ocurrido un problema al leer la matriz de Tags");
					logger.info(e.toString());
					return false;
				}
				  				 		 
		   System.out.println("Firma creada  correctamente");
		  
		return true; 
		  }else{
			  logger.severe("No se han introducido dos nombres para la creación de la firma");
			  return false;
			  }
	  }
}
