package extractor.caracteristicas;


import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.LogManager;
import java.util.logging.Logger;

import Utilities.ExecuteScript;

/**
 * Clase Extractor
 * Se encarga de toda la extración 
 * de características
 *
 * @author alejandro
 *
 */

public class Extractor {

	
	 static Logger logger = Logger.getLogger(Extractor.class.getName());
		
	 public static void main(String[] args){
		 
		  System.out.println("*-----------------------------------*");
		  System.out.println("*-----------------------------------*");
		  System.out.println("MODULO EXTRACCIÓN DE CARACTERISTICAS");
		  System.out.println("*-----------------------------------*");
		  System.out.println("*-----------------------------------*");
		  System.out.println("\n\n");
		  
		 System.out.println("Inicializando entorno R...");
		 //Variable que se encarga de ejecutar codigo en R
		 ExecuteScript script = new ExecuteScript();
		 System.out.println("Entorno R cargado correctamente");
		 
		 
		//Leyendo fichero de configuracion de la plataforma
		 System.out.println("Cargando fichero de configuracion del extractor..."); 
		//Variable que lee del fichero de propiedades
		 Properties config = new Properties();
		 //Directorio donde se encuentran los ficheros de propiedades
		 String path = new File("./Properties").getAbsolutePath();
		 
		 try {
				config.load(new FileInputStream(path+"/nucleo.properties"));
			} catch (FileNotFoundException e) {
				logger.info("Error al leer el archivo de propiedades: Archivo nucleo.properties no encontrado");
				logger.severe(e.toString());
				System.exit(0);
			} catch (IOException e) {
				logger.info("Error al cargar el archivo de propiedades nucleo.properties");
				logger.severe(e.toString());
				System.exit(0);
			}
		 
		 System.out.println("Fichero de propiedades cargado correctamente");		 
		 
		 //Variable que se encarga de crear la firma
		 CreateSign signCreator = new CreateSign(script);
		 
		 //Variable que creará la lista de palabaras funcionales
		 FunctionWordsCreator functionWordsCreator = new FunctionWordsCreator(script);
		 
		 //Variable que obtiene el vector de componentes principales
		 AnalisisPCA analisisPCA = new AnalisisPCA(script);
		
		 try{
			   LogManager.getLogManager().readConfiguration(new FileInputStream(path+"/logger.properties"));
		
		    	
		   }catch(Exception e){
			script.terminate();   
		    throw new RuntimeException("No se ha podido configurar el sistema de trazas: " +e.toString());
		   };
		   

		 //Guardo en una cadena de Strings los nombres pasados como argumentos
		 
		String [] nombres = args;
		
		//Variables config para la firma
		String num_firma =config.getProperty("num_firma");
		String token_min=config.getProperty("token_min");;
		String token_max=config.getProperty("token_max");;
		String box_plot=config.getProperty("plot_box");;
		String range_valor=config.getProperty("range_valor");;
		String num_outliers_allow=config.getProperty("num_outliers_allow");;
		//Variables de configuración para las palabras funcionales
		String num_func_words = config.getProperty("num_func_words");
		
		//Variables de configuracion para analisis PCA		
		String num_PCA = config.getProperty("num_PCA");
		
		//Variables de configuracion globales
		String tam_Firma = String.valueOf(Integer.parseInt(num_firma) + Integer.parseInt(num_func_words));
		
		//-------------FIN DE CARGA DE VARIABLES DE CONFIGURACION-------------------------------------------
		
		
		for(int i =0;i<nombres.length-1;i++){
			
			
			for(int j=i+1;j<nombres.length;j++){
				
				String [] autores = {nombres[i],nombres[j]};
				//Creación de la firma
				System.out.println("*---------------------------------------------------*");
				System.out.println("Procesando autores: "+nombres[i]+" y "+nombres[j]);				
				if(Integer.parseInt(num_firma)>0){
				if(!signCreator.createSign(num_firma,num_outliers_allow,range_valor,box_plot,token_min,token_max,autores)){					
					logger.severe("No se ha podido crear la firma de "+nombres[i]+"_"+nombres[j]+" correctamente");
					System.out.println("No se creó la firma de los sujetos "+nombres[i]+"_"+nombres[j]+"\nNo se continuara con el programa");
					System.exit(0);
				}
				}
				if(Integer.parseInt(num_func_words)>0){
					System.out.println("Creando la lista de palabras funcionales para:" +autores[0]+" y " +autores[1]);
					//Se ha creado correctamente la firma se pasa a la creación de la lista de palabras funcionales
					String[] listFunctionWords = null;
					try{
					listFunctionWords = functionWordsCreator.getListFunctionWords(num_func_words, autores);
					}catch(Exception e){
						logger.info("No se obtuvo la lista de palabras funcionales para los sujetos: "+nombres[i]+"_"+nombres[j]);
						logger.severe("Error al obtener la lista de palabras funcionales  para los sujetos: "+nombres[i]+"_"+nombres[j]+"\nInfo:"+e.toString());
						script.terminate();
						throw new RuntimeException("No se obtuvo la lista de palabras funcionales para los sujetos: "+nombres[i]+"_"+nombres[j]);
					}
					
						if(listFunctionWords == null || listFunctionWords.length==0){
							logger.info("No se obtuvo la lista de palabras funcionales para los sujetos: "+nombres[i]+"_"+nombres[j]);		
							script.terminate();
							throw new RuntimeException("No se obtuvo la lista de palabras funcionales para los sujetos: "+nombres[i]+"_"+nombres[j]);
						}
					}
					
								
				
				System.out.println("Lista de palabras funcionales creada correctamente");
				System.out.println("*---------------------------------------------------*");
				System.out.println("\n\n");
				
				System.out.println("*---------------------------------------------------*");
				System.out.println("Proceso de extración de componenentes principales");
				System.out.println("*---------------------------------------------------*");
			
				String [] sujetos  = {nombres[i],nombres[j]};
				analisisPCA.getPrincipalComponents(num_PCA,num_firma,num_func_words,tam_Firma,sujetos);
				System.out.println("Componentes principales obtenidos correctamente");
				System.gc();
			}
			
			
		}
		System.out.println("*---------------------------------------------------*");
		System.out.println("Proceso de extración de caracteristicas terminado");
		System.out.println("*---------------------------------------------------*");
		script.terminate();
	 }
}
