package extractor.caracteristicas;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import Utilities.CreateTagMatrix;
import Utilities.ExecuteScript;
import alex.tfg.freeling.ChartParser;
import alex.tfg.freeling.DepTxala;
import alex.tfg.freeling.HmmTagger;
import alex.tfg.freeling.LangIdent;
import alex.tfg.freeling.ListSentence;
import alex.tfg.freeling.ListSentenceIterator;
import alex.tfg.freeling.ListWord;
import alex.tfg.freeling.ListWordIterator;
import alex.tfg.freeling.Maco;
import alex.tfg.freeling.MacoOptions;
import alex.tfg.freeling.Nec;
import alex.tfg.freeling.Senses;
import alex.tfg.freeling.Sentence;
import alex.tfg.freeling.Splitter;
import alex.tfg.freeling.Tokenizer;
import alex.tfg.freeling.Ukb;
import alex.tfg.freeling.Util;
import alex.tfg.freeling.Word;
import analizador.Analyzer;


/**
 * Clase  FunctionWords,	
 * 
 * Permite obtener el conjunto de palabras "funcionales"
 * que tienen mayor aparicion en los textos
 * para 2 autores
 * @author alejandro
 *
 */
public class FunctionWordsCreator {
	
	 // Modify this line to be your FreeLing installation directory
	  private static final String FREELINGDIR = "/usr/local";
	  private static final String DATA = FREELINGDIR + "/share/freeling/";
	  private static final String LANG = "es";
	  
	  private LangIdent lgid;
	  private Tokenizer tk;
	  private Splitter sp;
	  private Maco mf;
	  private HmmTagger tg;
	  private ChartParser parser;
	  private DepTxala dep;
	  private Nec neclass;
	  private Senses sen; 
	  private Ukb dis;

	  
	  //Variable que sirve como sistema de trazas
	  private final static Logger logger = Logger.getLogger(Analyzer.class.getName());	  
	  
	 //Variable para ejecutar scrips en R
	  private ExecuteScript script;	  
	 //Variable que recorre los directorios del autor
	  private File directorio;
	  //Variable que guarda el path absoluto
	  private String path;
	  
	 //Variable que escribira la lista de palabras funcionales
	  BufferedWriter writer = null;
	  
	  /**
	   * Constructor clase FunctionWordsCreator,
	   * Inicializa variable script
	   * para ejecutar codigo en R
	   *
	   * @param script
	   */
	  public FunctionWordsCreator(ExecuteScript script){
		  this.script = script;
		  
		  //Inicializamos analizador de textos
		  System.loadLibrary( "freeling_javaAPI" ); 
		  Util.initLocale( "default" );
		  
		  // Create options set for maco analyzer.
		    // Default values are Ok, except for data files.
		    MacoOptions op = new MacoOptions( LANG );

		    op.setActiveModules(false, true, true, true, 
		                               true, true, true, 
		                               true, true, true);

		    op.setDataFiles(
		      "", 
		      DATA+LANG+"/locucions.dat", 
		      DATA + LANG + "/quantities.dat",
		      DATA + LANG + "/afixos.dat",
		      DATA + LANG + "/probabilitats.dat",
		      DATA + LANG + "/dicc.src",
		      DATA + LANG + "/np.dat",
		      DATA + "common/punct.dat");

		    // Create analyzers.
		    lgid = new LangIdent(DATA + "/common/lang_ident/ident-few.dat");

		    tk = new Tokenizer( DATA + LANG + "/tokenizer.dat" );
		   sp = new Splitter( DATA + LANG + "/splitter.dat" );
		    mf = new Maco( op );

		   tg = new HmmTagger( DATA + LANG + "/tagger.dat", true, 2 );
		    parser = new ChartParser(
		     DATA + LANG + "/chunker/grammar-chunk.dat" );
		    dep = new DepTxala( DATA + LANG + "/dep/dependences.dat",
		     parser.getStartSymbol() );
		  neclass = new Nec( DATA + LANG + "/nerc/nec/nec-ab-poor1.dat" );

		   sen = new Senses(DATA + LANG + "/senses.dat" ); // sense dictionary
		 dis = new Ukb( DATA + LANG + "/ukb.dat" ); // sense disambiguator
				
	  }
	  
	  
	  
	  public String[] getListFunctionWords(String numWords,String...nombres) throws Exception{
		
		  //Lista de function words
		  String [] listFunctionWords = null;
		  ListWord l=null;
		  ListSentence ls=null;
		  String Tag=null;
		  List<String> listaStopWords = new ArrayList<String>();
		  boolean repetir=false;
		  if(nombres.length==0 || nombres ==null){
			  throw new Exception("Lista de nombres para obtener lista de palabras funcionales vacía");
		  }else{
					
			  script.limpiarConsola();	
		  	  path = new File("").getAbsolutePath();
		  	do{
		  	//Vaciamos entorno R				  
		  	  
		  	  //Cargo librerias en el entorno R e inicializo variables
		  	  script.eval("library('tm')");	
		  	  //Cargo el numero de palabras funcionales
		  	  script.eval("numWords<-"+numWords);
			
		  	  
		  	  repetir = false;  
		  	  for(String nombre:nombres){
				  
				  //script.assign("autor", nombre);
				  //Primero obtenemos la lista de ficheros  que se encuentran en el autor
				  
				  directorio = new File(path+"/Documentos/"+nombre+"/results_Train");
				  if(!directorio.exists()){
					  logger.severe("Error, el directorio: " + "./Documentos/"+nombre+"/results_Train" + " no existe");
					  throw new Exception("Error, el directorio no existe");
				  }else{					  							  
						  //Ahora cargamos en el entorno R el nombre  del autor para cargar los documentos de entreno
						  script.assign("autor", nombre);
						  script.Reval(new File(path+"/ScriptR/loadAuthor.R"), "Documentos");		
						  
				  }		  
				  
			  }			  
		  	  script.Reval(new File(path+"/ScriptR/functionWords.R"), "Documentos");
			  listFunctionWords = script.getStringArray("function.words");
			  int contador = 0;
			  for(String Fword:listFunctionWords){
				  
				  	String Fword2=Fword.concat(".");
			    	// Extract the tokens from the line of text.
			    	l = tk.tokenize( Fword2 );

			      // Split the tokens into distinct sentences.
			    	 ls = sp.split( l, false );
				  
			         // Perform morphological analysis
			    	   	try{
			    	    	mf.analyze( ls );
			    	   	}catch(Exception e){
			    	   		System.out.println("Error:"+ e.toString());
			    	   	}
			    	   	
		    	      // Perform part-of-speech tagging.
		    	      tg.analyze( ls );
	
		    	      // Perform named entity (NE) classificiation.
		    	      neclass.analyze( ls );
	
		    	      sen.analyze( ls );
		    	      dis.analyze( ls );  	
			       
		    	      Long size = ls.size();
		    	      if(size>1){
		    	    	  logger.severe("Fallo al analizar sintacticamente la siguiente palabra funcional: "+Fword);
		    	    	  return null;
		    	      }else{
		    	    	  if(size!=0){
		    	    	  // get the analyzed words out of ls.
					      ListSentenceIterator sIt = new ListSentenceIterator(ls);
					      ListWordIterator wIt;
					      Sentence s;
					      Word w;
					      //Leo la frase
					      s = sIt.next();
					      wIt = new ListWordIterator(s);
					      //Leo la palabra
					      w = wIt.next();
					      //Ahora obtengo el Tag y comprueba que es una palabra funcional
					      Tag = w.getTag();
						      if(Tag.startsWith("N")||Tag.startsWith("V") || Tag.startsWith("A") |Tag.startsWith("Z") |Tag.startsWith("W")){
						    	  Fword = Fword.toLowerCase();
						    	  listaStopWords.add(Fword);
						    	  repetir = true;
						      }else{
						    	  if(Fword.equals(".")){
						    		 listFunctionWords[contador]= "\\.";
						    		  
						    	  }
						      }
					    
		    	    	  }else{
		    	    		  listaStopWords.add(Fword);
		    	    		  repetir = true;
		    	    	  }
		    	    	  
		    	      }   
		    	      contador++;
			  	}
			  	if(repetir){
			  	  script.limpiarConsola();
				  Iterator<String> it = listaStopWords.iterator();				 
				  script.eval("lista.stop.words<-vector()");
			      while(it.hasNext()){			      
			      script.eval("lista.stop.words<-c(lista.stop.words,\""+it.next()+"\")");
			      }
			  	}
			  
		  	  }while(repetir);
			 
		  	System.out.println("Lista de palabras funcionales:");
		  	script.eval(" print(function.words)");
			  //Guardo en un fichero la lista obtenida
			  
			  	//Compruebo que el directorio existe
			  	directorio = new File(path+"/Chooser/Function Words/"+nombres[0]+"-"+nombres[1]);
			  	directorio.mkdirs();
			  	//Compruebo si existe el ficheroy si es así lo elimino
			  	directorio = new File(path+"/Chooser/Function Words/"+nombres[0]+"-"+nombres[1]+"/functionWords.dat");
			  	if(directorio.exists()) directorio.delete();
			  	writer = new BufferedWriter(new FileWriter(new File(path+"/Chooser/Function Words/"+nombres[0]+"-"+nombres[1]+"/functionWords.dat")));
			  	for(int i=0;i<listFunctionWords.length;i++){
				  
				  writer.append(listFunctionWords[i]);
				  if(i!=listFunctionWords.length-1) writer.append("\n");
				  
			  	}
			  	writer.close();
			  return listFunctionWords;
		  }
		  		  
	  }

}
