package selector.caracteristicas;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import Utilities.FunctionWordsWeight;



/**
 * Clase FunctionWordsExtractor,
 * 
 * Permite obtener los pesos de las palabras
 * que aparecen en el fichero donde se
 * almacenan las "function words" seleccionadas
 * 
 * @author alejandro
 *
 */
public class FunctionWordsExtractor {
	
	  //Variable que sirve como sistema de trazas
	  private final static Logger logger = Logger.getLogger(FunctionWordsExtractor.class.getName());	 
	  
	  //Variable que calcula los pesos del doc
	  private FunctionWordsWeight pesos;
	  
	  //Pesos obtenidos del documento
	  List<Float> pesosGenerados = new ArrayList(); 
		 
	  private String path = new File("").getAbsolutePath(); 
	  
	  
	  public FunctionWordsExtractor(){
		  
		  System.out.println("Inicializando extractor de palabras funcionales");
		  pesos = new FunctionWordsWeight();
		  System.out.println("Extractor inicializado");
	  }
	  
	 	public List<Float> getPesosDoc(File doc,String ... nombres) throws Exception{
			  
	 		
		 	  String name =null;
			  
			  //Primero obtengo la firma del autor
			  //Compruebo si está creada
		 	  name = nombres[0] + "-" + nombres[1]; 
		 	  if(!new File(path+"/Chooser/Function Words/"+name+"/functionWords.dat").exists()){
		 		 name =  nombres[1] + "-" + nombres[0]; 
		 		 if(!new File(path+"/Chooser/Function Words/"+name+"/functionWords.dat").exists()){
		 			 
		 			throw new Exception("El  fichero con las palabras funcionales de los autores: "+nombres[0] +" y "+nombres[1]+" No existe.");
		 		 }
		 	  }
	 		
		 	  //Limpio buffer de posible basura
			  pesosGenerados.clear();
		
			  
			  		 
			  		//Variable donde guardo la firma
			  		List<String> sign = pesos.getFunctionWordsFromFile("Chooser/Function Words/"+name);
			  		
			  		pesosGenerados = pesos.CountTagger(doc,sign);
			 
			  		return pesosGenerados;
			  
		  }
	 	public List<String> getFunctionWords(String ... nombres) throws Exception{
	 		
	 		 String name =null;
			  
			  //Primero obtengo la firma del autor
			  //Compruebo si está creada
		 	  name = nombres[0] + "-" + nombres[1]; 
		 	  if(!new File(path+"/Chooser/Function Words/"+name+"/functionWords.dat").exists()){
		 		 name =  nombres[1] + "-" + nombres[0]; 
		 		 if(!new File(path+"/Chooser/Function Words/"+name+"/functionWords.dat").exists()){
		 			 
		 			throw new Exception("El  fichero con las palabras funcionales de los autores: "+nombres[0] +" y "+nombres[1]+" No existe.");
		 		 }
		 	  }
	 		
	 		return pesos.getFunctionWordsFromFile("Chooser/Function Words/"+name);
	 		
	 	}

}
