package selector.caracteristicas;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;



/**
 * Clase Selector,
 * Se encarga de seleccionar 
 * las características que iran 
 * al modulode analisis PCA  
 * @author alejandro
 *
 */
public class Selector {
	
	
	static Logger logger = Logger.getLogger(Selector.class.getName());

		
	//Variable que contiene todas las caracteristicas
	private List<Float> caracteristicas;
	
	//Variable que obtiene las pesos de las palabras funcionales
	FunctionWordsExtractor pesosFWordsExtractor;
	int num_function_words;
	//Variable que obtiene los pesos de la firma
	PesosFirmaExtractor pesosFirmaExtractor;
	int num_firma;
	
	public Selector(int num_firma,int num_function_words){
		
			
		this.caracteristicas= new ArrayList();
		if(num_function_words >0 )this.pesosFWordsExtractor = new FunctionWordsExtractor();
		if(num_firma>0)this.pesosFirmaExtractor = new PesosFirmaExtractor(num_firma);
	
		this.num_firma = num_firma;
		this.num_function_words = num_function_words;
	}
	
	public List<Float> getCaracteristicas(File doc, String ... nombres) throws Exception{
		
		
		//Primero compruebo que se hallan introducido nombres de autores (Al menos dos)
		
		if(nombres.length<2 || nombres==null){
			
			logger.severe("Error, no se han introducido dos  nombres de autores");
		}else{
			
			//Limpio de posible basura la variable que contendrá las características
			caracteristicas.clear();
			
		  String nombre;	  
		
					
					nombre = nombres[0]+"-"+nombres[1];
					System.out.println("Obteniendo caracteristicas de los autores " + nombres[0] + " y " + nombres[1]);
					try {
						if(num_firma>0)caracteristicas.addAll(pesosFirmaExtractor.getPesosDoc(doc, nombres[0],nombres[1]));
					} catch (Exception e) {
						logger.severe("Error al obtener los o máspesos de la firma del autor "+nombre+ "Razon: "+e.toString());
						throw new Exception("Error, no se pudo obtener los pesos de la firma del autor: "+nombre);
					}
					try {
						if(num_function_words>0)caracteristicas.addAll(pesosFWordsExtractor.getPesosDoc(doc, nombres[0],nombres[1]));
					} catch (Exception e) {
						logger.severe("Error al obtener los pesos de las palabras funcionales del autor "+nombre+ "Razon: "+e.toString());
						throw new Exception("Error, no se pudo obtener los pesos de las palabras funcionales Sdel autor: "+nombre);
					}
			
		}
		
		
		
		return caracteristicas;
	}
	
	public List<String> getFirma(String ... nombres) throws Exception{
		
		return pesosFirmaExtractor.getFirma(nombres);
	}
	
	public List<String> getFunctionWords(String ... nombres) throws Exception{
		
		return pesosFWordsExtractor.getFunctionWords(nombres);
	}
}
