#Modelo SVM
#Cargo las librerias que voy a utilizar
library(e1071)
library(rpart)
library(partykit)
library(ada)
library(randomForest)
library(nnet)
library(kernlab)

#PARA DEBUG----------------------------------------
#setwd("/home/alejandro/workspace/TFG-Authorship-Attribution/Chooser")
#---------------------------------------------------

#Uso variable para detectar errores
error <- 0
print("Cargando documentos de test...")
pesos.test <- read.table("Model/Test/pesos.dat", header=TRUE, sep="\t")
print("Carga OK")
print("Cargando documentos de train...") 
pesos.train<- read.table("Model/Train/pesos.dat", header=TRUE, sep="\t")
print("OK")
num_firma <- 41



#Modelo SVM
print("Optimizando parametros Gamma y Cost para modelado SVM...")
tuned <- tune.svm(Sujeto ~ ., data= pesos.train,gamma=10^(-6:1),cost=10^(-5:1))
#Mediante crossvalidation compruebo
#Cuales son los valores optimos del modelo
print(tuned)

svm.model <- svm(Sujeto ~ .,data = pesos.train,type="C-classification",kernel="radial",gamma=tuned$best.parameters$gamma,cost= tuned$best.parameters$cost,cross=5)
svm.pred <- predict(svm.model,pesos.test[,-num_firma],type="class")
print("Guardando modelo svm")
save(svm.model, file="Model/svm/svm.RData")
print("Modelo guardado")
table.svm<-table(pred=svm.pred,true=pesos.test[,num_firma])
print("Resultados del test del modelo SVM:")
print(table.svm)
filas <- dim(table.svm)[1]
columna <- dim(table.svm)[2]
if(filas == 1){
  #Si entra ha ocurrido algún error en el proceso de generación del modelo
  print("No ha sido posible generar el modelo de forma correcta")
  error <- 1
}else{
  errores.svm <- 0
  for(j in 1:filas){
    for(i in 1:columna){
      if(j == i){
        #No se hace nada
      }else{
        errores.svm <- errores.svm + table.svm[j,i]
      }
    }
  }
  
}


#Modelo Red Neuronal
print("Inicio de creacion del modelo de Red Neuronal")

tuned <- tune.nnet(Sujeto ~ ., data = pesos.train,size=seq(1,10,1),MaxNWts=10000)
print(tuned)
nnet.model <- nnet(Sujeto ~ ., data = pesos.train,size=tuned$best.parameters$size,decay=0,skip=TRUE,MaxNWts=10000,trace=FALSE,maxit=250)
print("Modelo creado, guardando modelo...")
nnet.pred <- predict(nnet.model,pesos.test[,-num_firma],type="class")
save(nnet.model, file="Model/neuronal/nnet.RData")
print("Modelo de Red Neuronal guardado")
table.nnet<-table(pred=nnet.pred,true=pesos.test[,num_firma])
print("Resultados del test del modelo Nnet:")
print(table.nnet)
filas <- dim(table.nnet)[1]
columna <- dim(table.nnet)[2]
if(filas == 1){
  #Si entra ha ocurrido algún error en el proceso de generación del modelo
  print("No ha sido posible generar el modelo de forma correcta")
  error <- 1
}else{
  errores.nnet <- 0
  for(j in 1:filas){
    for(i in 1:columna){
      if(j == i){
        #No se hace nada
      }else{
        errores.nnet <- errores.nnet + table.nnet[j,i]
      }
    }
  }
  
}



#Modelo Naive Bayes
bayes.model <- naiveBayes(Sujeto ~ ., data = pesos.train)
bayes.pred <- predict(bayes.model,pesos.test[,-num_firma],type="class")
save(bayes.model, file="Model/Bayes/bayes.RData")
table.bayes<-table(pred=bayes.pred,true=pesos.test[,num_firma])
print("Resultados del test del modelo Bayes:")
print(table.bayes)



if(errores.svm < errores.nnet){
  vector.error <- c("svm.model","nnet.model","bayes.model")
}else{
  vector.error <- c("nnet.model","svm.model","bayes.model")
}
if(error == 0){
  print("Modelos creados correctamente , FIN")
  
}
if(error == 1){
  print("No se han podido generar alguno/s de los modelos correctamente")
}
save(vector.error,file="Model/v_error/vector_error.RData")